from __future__ import unicode_literals

from django.core.exceptions import ValidationError
from django.conf import settings
from django.utils.text import force_text
from django.utils.translation import ugettext_lazy as _
import pytz


TIMEZONE_SESSION_KEY = getattr(
    settings, 'SUNDIAL_TIMEZONE_SESSION_KEY', '_timezone'
)


def set_session_timezone(session, zone):
    session[TIMEZONE_SESSION_KEY] = force_text(zone)


def get_session_timezone(session):
    return session.get(TIMEZONE_SESSION_KEY)


def coerce_timezone(zone):
    try:
        return pytz.timezone(zone)
    except pytz.UnknownTimeZoneError:
        raise ValidationError(
            _('Unknown timezone.'), code='invalid'
        )



from __future__ import unicode_literals

from django.utils import timezone

from .utils import get_session_timezone


class TimezoneMiddleware(object):
    """Middleware that sets the current timezone from a session stored value.

    Use ``sundial.utils.set_session_timezone()`` to store a user based value
    by registering a receiver on the ``django.contrib.auth.signals.user_logged_in``
    signal or override the ``get_timezone_from_request()`` method in a subclass
    to provide your own mechanism (e.g. GEOIP based)
    """

    def get_timezone_from_request(self, request):
        session = getattr(request, 'session', None)
        if session:
            return get_session_timezone(session)

    def process_request(self, request):
        zone = self.get_timezone_from_request(request)
        if zone:
            timezone.activate(zone)



from __future__ import unicode_literals

import django
from django.db import models
from django.utils.encoding import force_text
from django.utils.six import with_metaclass

from . import forms
from .utils import coerce_timezone


TimezoneFieldBase = type if django.VERSION >= (1, 8) else models.SubfieldBase


class TimezoneField(with_metaclass(TimezoneFieldBase, models.CharField)):
    default_max_length = 42

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('max_length', self.default_max_length)
        super(TimezoneField, self).__init__(*args, **kwargs)

    def from_db_value(self, value, connection, context):
        if value:
            value = coerce_timezone(value)
        return value

    def to_python(self, value):
        value = super(TimezoneField, self).to_python(value)
        if value:
            value = coerce_timezone(value)
        return value

    def validate(self, value, model_instance):
        super(TimezoneField, self).validate(force_text(value), model_instance)

    def run_validators(self, value):
        super(TimezoneField, self).run_validators(force_text(value))

    def get_prep_value(self, value):
        if value is not None:
            value = force_text(value)
        return value

    def deconstruct(self):
        name, path, args, kwargs = super(TimezoneField, self).deconstruct()
        if kwargs.get('max_length') == self.default_max_length:
            del kwargs['max_length']
        return name, path, args, kwargs

    def south_field_triple(self):
        """Provide a suitable description of this field for South."""
        from south.modelsinspector import introspector
        args, kwargs = introspector(self)
        if self.max_length == self.default_max_length:
            del kwargs['max_length']
        return 'sundial.fields.TimezoneField', args, kwargs

    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', forms.TimezoneField)
        kwargs.setdefault('choices_form_class', forms.TimezoneChoiceField)
        return super(TimezoneField, self).formfield(**kwargs)



from __future__ import unicode_literals

from datetime import datetime
from itertools import groupby

from django.utils.functional import lazy
from django.utils import six
from django.utils.translation import ugettext_lazy as _
import pytz

__all__ = ['COMMON_CHOICES', 'COMMON_GROUPED_CHOICES', 'ALL_CHOICES', 'ALL_GROUPED_CHOICES']


def _label_zone(zone):
    offset = datetime.now(pytz.timezone('/'.join(zone))).strftime('%z')
    return _("%(name)s (GMT%(offset)s)") % {
        'name': _(zone[-1].replace('_', ' ')),
        'offset': offset,
    }

_lazy_label_zone = lazy(_label_zone, six.text_type)


def _group_choices(zones):
    groups = groupby(six.moves.map(lambda z: z.split('/'), zones), lambda z: z[0])
    for group, regions in groups:
        choices = list(
            ('/'.join(region), _lazy_label_zone(region)) for region in regions
        )
        yield _(group), choices


def _flatten_choices(groups):
    for _group, choices in groups:
        # yield from choices
        for choice, value in choices:
            yield choice, value


class TimezoneChoices(object):
    def __init__(self, name, grouped=False):
        self.name = name
        self.grouped = grouped
        zones = getattr(pytz, "%s_timezones" % name)
        choices = _group_choices(zones)
        if not grouped:
            choices = _flatten_choices(choices)
        self._choices = list(choices)

    def __iter__(self):
        return iter(self._choices)

    def __getitem__(self, key):
        return self._choices[key]

    def deconstruct(self):
        kwargs = {'grouped': True} if self.grouped else {}
        return 'sundial.zones.TimezoneChoices', (self.name,), kwargs


COMMON_CHOICES = TimezoneChoices('common')
COMMON_GROUPED_CHOICES = TimezoneChoices('common', grouped=True)

ALL_CHOICES = TimezoneChoices('all')
ALL_GROUPED_CHOICES = TimezoneChoices('all', grouped=True)



from __future__ import unicode_literals

from django.forms import CharField, TypedChoiceField
from django.utils.encoding import force_text

from .utils import coerce_timezone


class TimezoneField(CharField):
    def to_python(self, value):
        if value in self.empty_values:
            return ''
        return coerce_timezone(value)

    def run_validators(self, value):
        return super(TimezoneField, self).run_validators(force_text(value))


class TimezoneChoiceField(TypedChoiceField):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('coerce', coerce_timezone)
        super(TimezoneChoiceField, self).__init__(*args, **kwargs)



from __future__ import unicode_literals

from setuptools import find_packages, setup

import sundial


with open('README.rst') as file_:
    long_description = file_.read()

setup(
    name='django-sundial',
    version=sundial.__version__,
    description='Django application providing database, form fields and middleware for timezone support.',
    long_description=long_description,
    url='https://github.com/charettes/django-sundial',
    author='Simon Charette.',
    author_email='charette.s+sundial@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords=['django timezone'],
    packages=find_packages(exclude=['tests', 'tests.*']),
    install_requires=['django>=1.6', 'pytz'],
    extras_require={
        'south': ['south'],
        'tests': ['tox'],
    },
)
